/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package com.rjconsultores.ventaboletos.dao.hibernate;

import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.List;

import org.apache.log4j.Logger;
import org.hibernate.Criteria;
import org.hibernate.Query;
import org.hibernate.SessionFactory;
import org.hibernate.criterion.Projections;
import org.hibernate.criterion.Restrictions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Repository;

import com.rjconsultores.ventaboletos.dao.CorridaDAO;
import com.rjconsultores.ventaboletos.entidad.ClaseServicio;
import com.rjconsultores.ventaboletos.entidad.Corrida;
import com.rjconsultores.ventaboletos.entidad.Corrida.Id;
import com.rjconsultores.ventaboletos.entidad.DiagramaAutobus;
import com.rjconsultores.ventaboletos.entidad.Marca;
import com.rjconsultores.ventaboletos.entidad.Parada;
import com.rjconsultores.ventaboletos.entidad.Ruta;
import com.rjconsultores.ventaboletos.utilerias.ApplicationProperties;

/**
 * 
 * @author rodrigo
 */
@Repository("corridaDAO")
public class CorridaHibernateDAO extends GenericHibernateDAO<Corrida, Corrida.Id> implements CorridaDAO {

	private static Logger log = Logger.getLogger(CorridaHibernateDAO.class);

	@Autowired
	public CorridaHibernateDAO(@Qualifier("sessionFactory") SessionFactory factory) {
		setSessionFactory(factory);
	}

	public Boolean existe(Id id) {

		Criteria c = makeCriteria();
		c.setProjection(Projections.rowCount());
		c.add(Restrictions.eq("id", id));

		Long cant = HibernateFix.count(c.list());

		return (cant > 0);

	}

	@Override
	public Corrida suscribir(Corrida entity) {

		if (ApplicationProperties.getInstance().gerarCampoEquivalenciaCorrida()) {
			Object o = this.getSession().createSQLQuery("SELECT VTABOL.CORRIDA_EQUIVALENCIA_SEQ.NEXTVAL FROM DUAL").uniqueResult();
			entity.setEquivalenciaId(o.toString());
		}

		Corrida corrida = super.suscribir(entity);
		this.getSession().flush();
		return corrida;
	}

	public List<Corrida> buscarPorAutobusCorridasFuturas(DiagramaAutobus diagramaAutobus) {
		String hql = " select co from Corrida co, Autobus ab "
			    + " where ab.autobusId = co.autobus.autobusId "
			    + " and ab.diagramaAutobus.diagramaautobusId = " + diagramaAutobus.getDiagramaautobusId()
			    + " and co.id.feccorrida > current_timestamp()";

		Query sq = getSession().createQuery(hql);
		List<Corrida> lsCorridas = sq.list();

		return lsCorridas;
	}

	public List<Corrida> buscarPorRolOperativoCorridasFuturas(DiagramaAutobus diagramaAutobus) {
		String hql = " select co from Corrida co, RolOperativo rol "
			+ " where rol.roloperativoId = co.rolOperativo.roloperativoId "
			+ " and rol.diagramaAutobus.diagramaautobusId = " + diagramaAutobus.getDiagramaautobusId()
		    + " and co.id.feccorrida > current_timestamp()";

		Query sq = getSession().createQuery(hql);
		List<Corrida> lsCorridas = sq.list();

		return lsCorridas;
	}

	public List<Corrida> buscarPorFecCorrida(Date value) {
		Criteria c = getSession().createCriteria(getPersistentClass());
		c.add(Restrictions.eq("activo", Boolean.TRUE));

		c.add(Restrictions.eq("id.feccorrida", value));

		return c.list();
	}

	public Long count(Ruta ruta) {
		Criteria c = this.makeCriteria();
		c.add(Restrictions.eq("ruta", ruta));
		c.add(Restrictions.eq("activo", Boolean.TRUE));
		c.setProjection(Projections.rowCount());

		return HibernateFix.count(c.list());

	}

	public List<Corrida> buscarPorId(Integer idCorrida) {
		Criteria c = getSession().createCriteria(getPersistentClass());
		c.add(Restrictions.eq("activo", Boolean.TRUE));

		c.add(Restrictions.eq("id.corridaId", idCorrida));

		return c.list();
	}

	// Metodo Ok.
	public List<Corrida> buscarFiltro(Parada ori, Parada des, Date hora, ClaseServicio cs) {
		Criteria c = getSession().createCriteria(getPersistentClass());
		c.add(Restrictions.eq("activo", Boolean.TRUE));

		// SimpleDateFormat sf = new
		// SimpleDateFormat("dd/MM/yyyy HH:mm:ss,SSS");
		// log.debug("Hora Corrida : " + sf.format(hora));
		// log.debug("Hora Corrida INI : " +
		// sf.format(getDiaGeracaoHoraIni(hora)));
		// log.debug("Hora Corrida FIM : " +
		// sf.format(getDiaGeracaoHoraFim(hora)));
		// log.debug("Origem       : " + ori.getParadaId());
		// log.debug("Destino      : " + des.getParadaId());
		// log.debug("Classe de Ser: " + cs.getClaseservicioId());

		c.add(Restrictions.eq("origem", ori));
		c.add(Restrictions.eq("destino", des));
		c.add(Restrictions.between("fechorsalida", getDiaGeracaoHoraIni(hora), getDiaGeracaoHoraFim(hora)));
		c.add(Restrictions.eq("claseServicio", cs));

		return c.list();
	}

	@Override
	public boolean count(Parada ori, Parada des, Date hora, ClaseServicio cs, boolean pisoExtra) {

		Criteria c = makeCriteria();
		c.setProjection(Projections.rowCount());
		c.add(Restrictions.eq("activo", Boolean.TRUE));
		c.add(Restrictions.eq("origem", ori));
		c.add(Restrictions.eq("destino", des));
		c.add(Restrictions.between("fechorsalida", getDiaGeracaoHoraIni(hora), getDiaGeracaoHoraFim(hora)));
		c.add(Restrictions.eq("claseServicio", cs));

		// Se for piso extra numPiso = 2
		if (pisoExtra) {
			// Corrida Piso Extra
			c.add(Restrictions.eq("numPiso", 2));
		} else {
			// num piso == null ou diferente de 2
			c.add(Restrictions.or(Restrictions.not(Restrictions.eq("numPiso", 2)), Restrictions.isNull("numPiso")));
		}

		Long cant = HibernateFix.count(c.list());

		return (cant > 0);
	}

	private Date getDiaGeracaoHoraIni(Date hora) {
		Date dataCorreta = new Date();

		// para pegar a hora e adicionar ao dia da geracao:
		GregorianCalendar gAux = new GregorianCalendar();
		gAux.setTime(hora);
		GregorianCalendar gcalendar = new GregorianCalendar();
		gcalendar.setTime(hora);
		gcalendar.set(Calendar.HOUR_OF_DAY, gAux.get(Calendar.HOUR_OF_DAY));
		gcalendar.set(Calendar.MINUTE, gAux.get(Calendar.MINUTE));
		gcalendar.set(Calendar.SECOND, 0);
		gcalendar.set(Calendar.MILLISECOND, 0);

		dataCorreta = gcalendar.getTime();

		return dataCorreta;
	}

	private Date getDiaGeracaoHoraFim(Date hora) {
		Date dataCorreta = new Date();

		// para pegar a hora e adicionar ao dia da geracao:
		GregorianCalendar gAux = new GregorianCalendar();
		gAux.setTime(hora);
		GregorianCalendar gcalendar = new GregorianCalendar();
		gcalendar.setTime(hora);
		gcalendar.set(Calendar.HOUR_OF_DAY, gAux.get(Calendar.HOUR_OF_DAY));
		gcalendar.set(Calendar.MINUTE, gAux.get(Calendar.MINUTE));
		gcalendar.set(Calendar.SECOND, 0);
		gcalendar.set(Calendar.MILLISECOND, 59);

		dataCorreta = gcalendar.getTime();

		return dataCorreta;
	}

	@Override
	public List<Corrida> buscarDistinct(Parada ori, Parada des, Date horaSalidaInicio, Date horaSalidaFin, ClaseServicio cs, Integer corridaId) {
		StringBuilder hql = new StringBuilder("");
		hql.append("select distinct ");
		hql.append("    new com.rjconsultores.ventaboletos.entidad.Corrida(");
		hql.append("	c.marca, ");
		hql.append("	c.id.corridaId, ");
		hql.append("	c.origem, ");
		hql.append("	c.destino, ");
		hql.append("	c.claseServicio, ");
		hql.append("	c.corridaCtrl.horario) ");
		hql.append("from ");
		hql.append("	Corrida c ");
		hql.append("where ");
		hql.append("	1 = 1 ");
		if (ori != null) {
			hql.append("	and c.origem = :origen ");
		}
		if (des != null) {
			hql.append("	and c.destino = :destino");
		}
		if (cs != null) {
			hql.append("	and c.claseServicio = :claseServicio ");
		}
		if (corridaId != null) {
			hql.append("	and c.id.corridaId = :corridaId ");
		}
		if (horaSalidaInicio != null) {

			hql.append("	and c.corridaCtrl.horario >= :horaSalidaInicio");
		}
		if (horaSalidaFin != null) {

			hql.append("	and c.corridaCtrl.horario <= :horaSalidaFin");
		}

		hql.append("  order by c.id.corridaId,c.corridaCtrl.horario");

		Query query = getSession().createQuery(hql.toString());// .setResultTransformer(new
																// AliasToBeanResultTransformer(Corrida.class));
		if (ori != null) {
			query.setParameter("origen", ori);
		}
		if (des != null) {
			query.setParameter("destino", des);
		}
		if (cs != null) {
			query.setParameter("claseServicio", cs);
		}
		if (corridaId != null) {
			query.setParameter("corridaId", corridaId);
		}

		if (horaSalidaInicio != null) {

			query.setParameter("horaSalidaInicio", horaSalidaInicio);
		}
		if (horaSalidaFin != null) {
			query.setParameter("horaSalidaFin", horaSalidaFin);

		}

		return query.list();
	}

	@Override
	public boolean existe(Ruta ruta, Date fecCorrida, Date horaCorrida, ClaseServicio claseServicio, Marca marca, boolean pisoExtra) {
		Calendar soloFecha = Calendar.getInstance();
		soloFecha.setTime(fecCorrida);
		soloFecha.set(Calendar.HOUR, 0);
		soloFecha.set(Calendar.MINUTE, 0);
		soloFecha.set(Calendar.SECOND, 0);
		soloFecha.set(Calendar.MILLISECOND, 0);

		Criteria c = makeCriteria();
		c.setProjection(Projections.rowCount());
		c.add(Restrictions.eq("ruta", ruta));
		c.add(Restrictions.eq("id.feccorrida", soloFecha.getTime()));
		c.add(Restrictions.eq("claseServicio", claseServicio));
		c.add(Restrictions.eq("marca", marca));

		// Se for piso extra numPiso = 2
		if (pisoExtra) {
			// Corrida Piso Extra
			c.add(Restrictions.eq("numPiso", 2));
		} else {
			// num piso == null ou diferente de 2
			c.add(Restrictions.or(Restrictions.not(Restrictions.eq("numPiso", 2)), Restrictions.isNull("numPiso")));
		}

		Criteria cCorridaCtrl = c.createCriteria("corridaCtrl");
		cCorridaCtrl.add(Restrictions.eq("horario", horaCorrida));

		Long cant = HibernateFix.count(c.list());

		return (cant > 0);
	}
}
