/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package com.rjconsultores.ventaboletos.service.impl;

import com.rjconsultores.ventaboletos.dao.CategoriaCtrlDAO;
import com.rjconsultores.ventaboletos.dao.CategoriaDAO;
import com.rjconsultores.ventaboletos.entidad.CategoriaClase;
import com.rjconsultores.ventaboletos.entidad.CategoriaCorrida;
import com.rjconsultores.ventaboletos.entidad.CategoriaCtrl;
import com.rjconsultores.ventaboletos.entidad.CategoriaDescuento;
import com.rjconsultores.ventaboletos.entidad.CategoriaMarca;
import com.rjconsultores.ventaboletos.entidad.CategoriaMercado;
import com.rjconsultores.ventaboletos.entidad.CategoriaPeriodo;
import com.rjconsultores.ventaboletos.service.CategoriaCtrlService;
import com.rjconsultores.ventaboletos.utilerias.UsuarioLogado;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.List;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

/**
 *
 * @author Administrador
 */
@Service("categoriaCtrlService")
public class CategoriaCtrlServiceImpl implements CategoriaCtrlService {

    public static final int CATEGORIA_ADULTO = 1;
    @Autowired
    private CategoriaCtrlDAO categoriaCtrlDAO;
    @Autowired
    private CategoriaDAO categoriaDAO;

    public List<CategoriaCtrl> obtenerTodos() {
        return categoriaCtrlDAO.obtenerTodos();
    }

    public CategoriaCtrl obtenerID(Integer id) {
        return categoriaCtrlDAO.obtenerID(id);
    }

    @Transactional
    public CategoriaCtrl suscribir(CategoriaCtrl entidad) {
        entidad.setUsuarioId(UsuarioLogado.getUsuarioLogado().getUsuarioId());
        entidad.setFecmodif(Calendar.getInstance().getTime());
        entidad.setActivo(Boolean.TRUE);

        checarCategoriaAdulto(entidad);

        return categoriaCtrlDAO.suscribir(entidad);
    }

    /**
     * Verifica se existe la categoria adulto. Sino existe, la adicciona.
     * @param categoriaCtrl 
     */
    private void checarCategoriaAdulto(CategoriaCtrl categoriaCtrl) {
        boolean existeAdulto = false;
        List<CategoriaDescuento> lsCategorias = new ArrayList<CategoriaDescuento>();
        lsCategorias.addAll(categoriaCtrl.getCategoriaDescuentoList());
        
        for (CategoriaDescuento cd : categoriaCtrl.getCategoriaDescuentoList()) {
            if (cd.getCategoria().getCategoriaId().equals(CATEGORIA_ADULTO)) {
                existeAdulto = true;
            }
        }
        if (!existeAdulto) {
            CategoriaDescuento categoriaDescuentoAdulto = this.getCategoriaDescuentoAdulto(categoriaCtrl);
            lsCategorias.add(categoriaDescuentoAdulto);
        }
        categoriaCtrl.setCategoriaDescuentoList(lsCategorias);
    }

    @Transactional
    public CategoriaCtrl actualizacion(CategoriaCtrl entidad) {
        entidad.setUsuarioId(UsuarioLogado.getUsuarioLogado().getUsuarioId());
        entidad.setFecmodif(Calendar.getInstance().getTime());
        entidad.setActivo(Boolean.TRUE);

        return categoriaCtrlDAO.actualizacion(entidad);
    }

    @Transactional
    public void borrar(CategoriaCtrl entidad) {

        for (CategoriaClase cc : entidad.getCategoriaClaseList()) {
            cc.setUsuarioId(UsuarioLogado.getUsuarioLogado().getUsuarioId());
            cc.setFecmodif(Calendar.getInstance().getTime());
            cc.setActivo(Boolean.FALSE);
        }
        for (CategoriaCorrida cc : entidad.getCategoriaCorridaList()) {
            cc.setUsuarioId(UsuarioLogado.getUsuarioLogado().getUsuarioId());
            cc.setFecmodif(Calendar.getInstance().getTime());
            cc.setActivo(Boolean.FALSE);
        }
        for (CategoriaDescuento cc : entidad.getCategoriaDescuentoList()) {
            cc.setUsuarioId(UsuarioLogado.getUsuarioLogado().getUsuarioId());
            cc.setFecmodif(Calendar.getInstance().getTime());
            cc.setActivo(Boolean.FALSE);
        }
        for (CategoriaMarca cc : entidad.getCategoriaMarcaList()) {
            cc.setUsuarioId(UsuarioLogado.getUsuarioLogado().getUsuarioId());
            cc.setFecmodif(Calendar.getInstance().getTime());
            cc.setActivo(Boolean.FALSE);
        }
        for (CategoriaMercado cc : entidad.getCategoriaMercadoList()) {
            cc.setUsuarioId(UsuarioLogado.getUsuarioLogado().getUsuarioId());
            cc.setFecmodif(Calendar.getInstance().getTime());
            cc.setActivo(Boolean.FALSE);
        }

        entidad.setUsuarioId(UsuarioLogado.getUsuarioLogado().getUsuarioId());
        entidad.setFecmodif(Calendar.getInstance().getTime());
        entidad.setActivo(Boolean.FALSE);


        categoriaCtrlDAO.actualizacion(entidad);
    }

    private CategoriaDescuento getCategoriaDescuentoAdulto(CategoriaCtrl categoriaCtrl) {
        CategoriaDescuento cd = new CategoriaDescuento();
        cd.setActivo(Boolean.TRUE);
        cd.setCantautorizada(50);
        cd.setCategoria(categoriaDAO.obtenerID(CATEGORIA_ADULTO));
        cd.setDescuentoimporte(BigDecimal.ZERO);
        cd.setDescuentoporc(BigDecimal.ZERO);

        CategoriaPeriodo cp = new CategoriaPeriodo();
        cp.setActivo(Boolean.TRUE);
        cp.setCategoriaDescuento(cd);

        Calendar datFin = Calendar.getInstance();
        datFin.set(2300, 11, 31);

        Calendar datInicio = Calendar.getInstance();
        datInicio.set(2000, 11, 31);

        cp.setFecfin(datFin.getTime());
        cp.setFecinicio(datInicio.getTime());

        cp.setFecmodif(new java.util.Date());
        cp.setUsuarioId(UsuarioLogado.getUsuarioLogado().getUsuarioId());

        List<CategoriaPeriodo> lsCatPeriodo = new ArrayList<CategoriaPeriodo>();
        lsCatPeriodo.add(cp);

        cd.setCategoriaPeriodoList(lsCatPeriodo);
        cd.setCategoriaCtrl(categoriaCtrl);
        cd.setFecmodif(new java.util.Date());
        cd.setUsuarioId(UsuarioLogado.getUsuarioLogado().getUsuarioId());


        return cd;
    }
}
