package com.rjconsultores.ventaboletos.service.impl;

import com.rjconsultores.ventaboletos.entidad.TarifaOficial;
import java.util.List;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.rjconsultores.ventaboletos.dao.SeguroKmDAO;
import com.rjconsultores.ventaboletos.dao.SeguroTarifaDAO;
import com.rjconsultores.ventaboletos.dao.TarifaOficialDAO;
import com.rjconsultores.ventaboletos.entidad.VigenciaTarifa;
import com.rjconsultores.ventaboletos.exception.BusinessException;
import com.rjconsultores.ventaboletos.service.TarifaOficialService;
import com.rjconsultores.ventaboletos.utilerias.UsuarioLogado;

@Service("tarifaOficialService")
public class TarifaOficialServiceImpl implements TarifaOficialService {

    @Autowired
    private TarifaOficialDAO tarifaOficialDAO;
    @Autowired
    private SeguroKmDAO seguroKmDAO;
    @Autowired
    private SeguroTarifaDAO seguroTarifaDAO;

    @Override
    @Transactional
    public Integer gerarTarifaPorCoeficiente(Integer rutaId, Integer orgaoConcedenteId) {
        return tarifaOficialDAO.gerarTarifaPorCoeficiente(rutaId, UsuarioLogado.getUsuarioLogado().getUsuarioId(), orgaoConcedenteId);
    }

    @Override
    @Transactional
    public Integer atualizarTarifaPorCoeficiente(Integer rutaId, Integer orgaoConcedenteId) {
        return tarifaOficialDAO.atualizarTarifaCoeficiente(rutaId, UsuarioLogado.getUsuarioLogado().getUsuarioId(), orgaoConcedenteId);
    }

    @Override
    @Transactional
    public void copiarParaTarifa(VigenciaTarifa vigenciaTarifa) {
        tarifaOficialDAO.copiarParaTarifa(vigenciaTarifa, UsuarioLogado.getUsuarioLogado().getUsuarioId());
    }

    @Override
    @Transactional(rollbackFor = BusinessException.class)
    public Integer gerarAtualizarTarifaPorCoeficiente(Integer rudaId, Integer orgaoConcedenteId) throws BusinessException {

        if (orgaoConcedenteId == null) {
            throw new BusinessException("TarifaOficialServiceImpl.msg.validacion.orgaoObligatorio");
        }

        Integer qtd = gerarTarifaPorCoeficiente(rudaId, orgaoConcedenteId);
        qtd += atualizarTarifaPorCoeficiente(rudaId, orgaoConcedenteId);

        return qtd;
    }

    @Override
    @Transactional
    public void atualizarTaxaEmbarque(Integer rutaId, Integer orgaoConcedenteId) {
        tarifaOficialDAO.atualizarTaxaEmbarque(rutaId, UsuarioLogado.getUsuarioLogado().getUsuarioId(), orgaoConcedenteId);
    }

    @Override
    @Transactional(rollbackFor = BusinessException.class)
    public void atualizarSeguroPorKm(Integer rutaId, Integer orgaoId) throws BusinessException {

        if (orgaoId == null) {
            throw new BusinessException("TarifaOficialServiceImpl.msg.validacion.orgaoObligatorio");
        }

        tarifaOficialDAO.atualizarSeguroPorKm(rutaId, orgaoId, UsuarioLogado.getUsuarioLogado().getUsuarioId());

    }

    @Override
    @Transactional(rollbackFor = BusinessException.class)
    public void atualizarSeguroPorTarifa(Integer rutaId, Integer orgaoId) throws BusinessException {
        if (orgaoId == null) {
            throw new BusinessException("TarifaOficialServiceImpl.msg.validacion.orgaoObligatorio");
        }

        tarifaOficialDAO.atualizarSeguroPorTarifa(rutaId, orgaoId, UsuarioLogado.getUsuarioLogado().getUsuarioId());
    }

    @Override
    @Transactional(rollbackFor = BusinessException.class)
    public void atualizarSeguro(Integer rutaId, Integer orgaoId) throws BusinessException {
        if (orgaoId == null) {
            throw new BusinessException("TarifaOficialServiceImpl.msg.validacion.orgaoObligatorio");
        }

        if (seguroKmDAO.existe(orgaoId)) {
            atualizarSeguroPorKm(rutaId, orgaoId);
        } else if (seguroTarifaDAO.existe(orgaoId)) {
            atualizarSeguroPorTarifa(rutaId, orgaoId);
        }
    }

    @Override
    @Transactional
    public void aplicarArredondamentoTarifa(Integer orgaoConcedenteId) {
        tarifaOficialDAO.aplicarArredondamentoTarifa(orgaoConcedenteId, UsuarioLogado.getUsuarioLogado().getUsuarioId());
    }

    public List<TarifaOficial> obtenerTodos() {
        return tarifaOficialDAO.obtenerTodos();
    }

    public TarifaOficial obtenerID(Integer id) {
return tarifaOficialDAO.obtenerID(id);
    }

    @Transactional
    public TarifaOficial suscribir(TarifaOficial entidad) {
        return tarifaOficialDAO.suscribir(entidad);
    }

    @Transactional
    public TarifaOficial actualizacion(TarifaOficial entidad) {
        return tarifaOficialDAO.actualizacion(entidad);
    }

    @Transactional
    public void borrar(TarifaOficial entidad) {
        tarifaOficialDAO.borrar(entidad);
    }
}
